package com.tfg.api.projecte_tfg.builder;

import com.tfg.api.projecte_tfg.enums.Action;
import com.tfg.api.projecte_tfg.enums.Member;
import com.tfg.api.projecte_tfg.enums.Status;
import com.tfg.api.projecte_tfg.model.*;
import com.tfg.api.projecte_tfg.rest.object.*;
import com.tfg.api.projecte_tfg.service.FollowGrupService;
import com.tfg.api.projecte_tfg.service.SCTrackService;
import com.tfg.api.projecte_tfg.util.DateUtils;
import com.tfg.api.projecte_tfg.util.StringUtils;

import javax.ejb.EJB;
import javax.ejb.LocalBean;
import javax.ejb.Stateless;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Created by marc on 22/04/14.
 */

@Stateless
@LocalBean
public class GrupBuilder {

    @EJB
    private UserBuilder userBuilder;

    @EJB
    private GrupBuilder grupBuilder;
    @EJB
    private SouncloudBuilder souncloudBuilder;

    @EJB
    private SCTrackService scTrackService;

    @EJB
    private FollowGrupService followGrupService;

    public GrupTO error(String s,Long idCurrent) {
        return new GrupTO(ModelJSON.ERROR,s,idCurrent);
    }

    public Grup create(String name, String description, String founded, StyleMusic styleMusica, ImageUser imageUser) throws ParseException {

        Date f = DateUtils.create(founded);

        return new Grup(name,styleMusica,description,f,true,imageUser);
    }

    public GrupTO createTO(Grup grup, Long idcurrent, List<GrupMember> lu,List<EventTO> eventTOList) {

        if(grup == null){
            return null;
        }

        String founded = DateUtils.Date(grup.getFounded());
        String styleMusic = grup.getStyleMusic().toString();

        Member member = Member.NOT_MEMBER;

        List<MemberTO> users = new ArrayList<MemberTO>(0);
        if(lu != null){

            users = new ArrayList<MemberTO>(lu.size());
            for(GrupMember grup_member : lu){

                if(grup_member.getStatus().equals(Status.OK) || grup_member.getStatus().equals(Status.PENDING)){

                    //member
                    if(grup_member.getUser().getId().equals(idcurrent)){
                        member = grup_member.getTypemember();
                    }

                    MemberTO u = grupBuilder.createMemberTO(grup_member, idcurrent);
                    users.add(u);

                }
            }
        }

        Boolean follow = false;

        FollowGrup followGrup = followGrupService.match(idcurrent, grup.getId());

        if(followGrup != null){
            follow = followGrup.getFollow() == null ? false : followGrup.getFollow();

        }

        byte[] img = null;
        if(grup.getImage() != null){
            img = grup.getImage().getImagen();
        }

        List<SCTrack> list = scTrackService.getListByGrup(grup.getId(),5,0l);
        List<SCTrackTO> scTrackTOList = souncloudBuilder.createListTO(list,idcurrent);

        Long friendsFollow = followGrupService.getFriendsFollow(grup.getId(),idcurrent);

        return new GrupTO(idcurrent,grup.getId(), StringUtils.ConvertirPrimeraLetraEnMayuscula(grup.getNom()),
                founded, styleMusic, grup.getDescription(),
                member,follow, users, Action.GRUP,img,eventTOList,scTrackTOList,friendsFollow);
    }

    private MemberTO createMemberTO(GrupMember grup_member, Long idcurrent) {

        String name = StringUtils.ConvertirPrimeraLetraEnMayuscula(grup_member.getUser().getNom().trim() +
                " " + grup_member.getUser().getCognoms().trim());

        MemberTO memberTO = new MemberTO(grup_member.getUser().getId(), name,
                grup_member.getTypemember(), grup_member.getStatus(), grup_member.getRol());

        return memberTO;

    }

    public List<GrupTO> createListGrupTO(List<GrupMember> l){

        List<GrupTO> grups = new ArrayList<GrupTO>(l.size());

        for(GrupMember grup_member : l){

            if(grup_member.getStatus().equals(Status.OK)){

                grups.add(new GrupTO());
            }
        }

        return grups;

    }

    public List<GrupMember> makeListMembersBasic(List<User> members,Grup grup) {

        try{
            List<GrupMember> l = new ArrayList<GrupMember>(members.size());

            for(User u : members)
                l.add(new GrupMember(u, grup, Status.PENDING, Member.BASIC,""));

            return l;
        } catch(Exception e){
            return null;
        }
    }

    public GrupMember createGrupMember(Grup grup, User u, String rol,Member typemember) {

        GrupMember grupMember = new GrupMember(u, grup, Status.PENDING, typemember, rol);

        return grupMember;
    }

    public void doFollowGrup(User user, Grup grup){
        FollowGrup followGrup = followGrupService.match(user.getId(),
                grup.getId());

        if(followGrup == null){
            followGrup = new FollowGrup(grup,user,true);
            followGrupService.save(followGrup);
        }else{
            followGrup.setFollow(true);
            followGrupService.update(followGrup);
        }
    }

    public ListTO<GrupTO> ListGrupTOLite(List<Grup> list, Long currentId) {

        try{
        if(list == null){return null;}

        List<GrupTO> grupTOs = new ArrayList<GrupTO>(list.size());

        for(Grup g : list){
            grupTOs.add(createTO(g,currentId,null,null));
        }

        ListTO<GrupTO> grupTOListTO = new ListTO<GrupTO>(currentId);
        grupTOListTO.setList(grupTOs);

        return grupTOListTO;
        }catch(Exception e){
            return null;
        }
    }

    public GrupTO createTOSimple(Grup grup, Long idcurrent, List<GrupMember> lu,List<EventTO> eventTOList) {

        if(grup == null){
            return null;
        }

        String founded = DateUtils.Date(grup.getFounded());
        String styleMusic = grup.getStyleMusic().toString();

        Member member = Member.NOT_MEMBER;

        List<MemberTO> users = null;

        Boolean follow = false;

        FollowGrup followGrup = followGrupService.match(idcurrent, grup.getId());

        if(followGrup != null){
            follow = followGrup.getFollow() == null ? false : followGrup.getFollow();
        }

        byte[] img = null;

        List<SCTrack> list = null;
        List<SCTrackTO> scTrackTOList = null;

        Long friendsFollow = followGrupService.getFriendsFollow(grup.getId(),idcurrent);

        return new GrupTO(idcurrent,grup.getId(), StringUtils.ConvertirPrimeraLetraEnMayuscula(grup.getNom()),
                founded, styleMusic, grup.getDescription(),
                member,follow, users, Action.GRUP,img,eventTOList,scTrackTOList,friendsFollow);
    }
}
