package com.tfg.api.projecte_tfg.builder;

import com.tfg.api.projecte_tfg.enums.*;
import com.tfg.api.projecte_tfg.model.*;
import com.tfg.api.projecte_tfg.rest.object.InfoUserTO;
import com.tfg.api.projecte_tfg.rest.object.ModelJSON;
import com.tfg.api.projecte_tfg.rest.object.UserTO;
import com.tfg.api.projecte_tfg.service.FriendService;
import com.tfg.api.projecte_tfg.service.InfoUserService;
import com.tfg.api.projecte_tfg.service.UserService;
import com.tfg.api.projecte_tfg.util.DateUtils;
import com.tfg.api.projecte_tfg.util.StringUtils;

import javax.ejb.EJB;
import javax.ejb.LocalBean;
import javax.ejb.Stateless;
import java.text.ParseException;

/**
 * Created by marc on 26/03/14.
 */

@Stateless
@LocalBean
public class InfoUserBuilder {

    @EJB
    private UserBuilder userBuilder;

    @EJB
    private AuthBuilder authBuilder;

    @EJB
    private UserService us;

    @EJB
    private InfoUserService ius;

    @EJB
    private FriendService friendService;

    @EJB
    private PrivacityBuilder privacityBuilder;

    public InfoUserTO error(String s) {
        return new InfoUserTO(ModelJSON.ERROR, s,null,null,null,null,null,null
                ,null,null,null,null,null,null,null,null,false);
    }

    public InfoUserTO createOK(InfoUser iu,Auth au,Long idcurrent){

        Boolean modal = false;

        Friend f  = friendService.match(idcurrent,iu.getUser().getId());

        if(iu.getBirthday() == null || iu.getSex() == null || iu.getCity() == null){
            modal = true;
        }

        Relation r = Relation.NO_FRIEND;
        if(f == null && idcurrent.equals(iu.getUser().getId())){
            r = Relation.OWN;
        }else if(f != null){

            if(f.getStatus().equals(Status.OK)){
                r =  Relation.FRIEND;
            }else{
                r =  Relation.NO_FRIEND;
            }
        }

        Person sex = null;
        if(privacityBuilder.mach(r,iu.getSexPrivacity())){
            sex = iu.getSex();
        }

        String birthday = null;
        if(privacityBuilder.mach(r,iu.getBirthdayPrivacity())){
            birthday = iu.getBirthday() == null ? null : DateUtils.Date(iu.getBirthday());
        }

        String country = null;
        if(privacityBuilder.mach(r,iu.getCountryPrivacity())){
            country = iu.getCountry();
        }

        String city = null;
        if(privacityBuilder.mach(r,iu.getCityPrivacity())){
            city = iu.getCity();
        }

        String phone = null;
        if(privacityBuilder.mach(r,iu.getPhonePrivacity())){
            phone = iu.getPhone();
        }

        UserTO user = userBuilder.createUserTO(au.getUser(),idcurrent);

        return new InfoUserTO(ModelJSON.OK, ModelJSON.MSG_EMPTY,idcurrent,
                iu.getId(),sex, birthday, country,city,phone, user,
                iu.getSexPrivacity(), iu.getBirthdayPrivacity(),
                iu.getCountryPrivacity(),iu.getCityPrivacity(),
                iu.getPhonePrivacity(),Action.USER,modal);
    }

    public InfoUser defauldInfoUser(User u,Auth auth) {
        return new InfoUser(null,null,null, null, null, u, Privacity.FRIENDS,Privacity.PUBLIC,
                Privacity.PUBLIC,Privacity.FRIENDS,Privacity.FRIENDS,auth == null ? null : auth.getEmail(),Privacity.PRIVATE,null);
    }

    public InfoUser edit(InfoUser iu, User u, String name, String secondname, String birthday, String birthdayPrivacity,
                         String sex, String sexPrivacity, String city, String cityPrivacity, String country,
                         String countryPrivacity, String description, ImageUser imageUser) throws ParseException {

        if(!StringUtils.isEmptyChar(name)){
            u.setNom(name);
        }

        if(!StringUtils.isEmptyChar(secondname)){
            u.setCognoms(secondname);
        }

        if(description != null){
            u.setDescription(description);
        }

        if(!StringUtils.isEmptyChar(birthday)){
            iu.setBirthday(DateUtils.create(birthday));
        }

        if(!StringUtils.isEmptyChar(birthdayPrivacity)){

            Privacity p = privacityBuilder.privacity(birthdayPrivacity);
            iu.setBirthdayPrivacity(p);
        }


        if(!StringUtils.isEmptyChar(sex)){

            if(Person.FEMALE.toString().equals(sex)){
                iu.setSex(Person.FEMALE);
            }else if(Person.MALE.toString().equals(sex)){
                iu.setSex(Person.MALE);
            }
        }

        if(!StringUtils.isEmptyChar(sexPrivacity)){

            Privacity p = privacityBuilder.privacity(sexPrivacity);
            iu.setSexPrivacity(p);
        }

        if(!StringUtils.isEmptyChar(city)){

            iu.setCity(city);
        }

        if(!StringUtils.isEmptyChar(cityPrivacity)){

            Privacity p = privacityBuilder.privacity(cityPrivacity);
            iu.setSexPrivacity(p);
        }

        if(!StringUtils.isEmptyChar(country)){

            iu.setCountry(country);
        }

        if(!StringUtils.isEmptyChar(countryPrivacity)){

            Privacity p = privacityBuilder.privacity(countryPrivacity);
            iu.setSexPrivacity(p);
        }

        if(imageUser != null){
            iu.setImage(imageUser);
        }

        ius.update(iu);
        us.update(u);

        return iu;
    }

    public ImageUser getPhoto(User u) {

        InfoUser iu = ius.getByIdUser(u.getId());

        if(iu == null){return null;}
        return iu.getImage();
    }
}
