package com.tfg.api.projecte_tfg.builder;

import com.tfg.api.projecte_tfg.enums.Status;
import com.tfg.api.projecte_tfg.enums.TypeNotifications;
import com.tfg.api.projecte_tfg.model.FollowGrup;
import com.tfg.api.projecte_tfg.model.GrupMember;
import com.tfg.api.projecte_tfg.model.Notifications;
import com.tfg.api.projecte_tfg.rest.object.*;
import com.tfg.api.projecte_tfg.service.FollowGrupService;
import com.tfg.api.projecte_tfg.service.Grup_UserService;
import com.tfg.api.projecte_tfg.util.DateUtils;
import com.tfg.api.projecte_tfg.util.StringUtils;

import javax.ejb.EJB;
import javax.ejb.LocalBean;
import javax.ejb.Stateless;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by marc on 28/04/14.
 */

@Stateless
@LocalBean
public class NotificationsBuilder {

    @EJB
    private UserBuilder userBuilder;

    @EJB
    private GrupBuilder grupBuilder;
    @EJB
    private Grup_UserService grup_userService;

    @EJB
    private FollowGrupService followGrupService;

    public ListNotificationsTO<NotificationsTO> makeListTO(List<Notifications> notificationsList,Long currentid,Long currentPage,Long nPage) {

        ListNotificationsTO<NotificationsTO> list = new ListNotificationsTO<NotificationsTO>(currentid,currentPage,nPage);
        List<NotificationsTO> notificationsTOs = new ArrayList<NotificationsTO>(notificationsList.size());

        for (Notifications n : notificationsList){

            NotificationsTO nTO = getNotificationsTO(currentid, n);

            notificationsTOs.add(nTO);
        }

        list.setList(notificationsTOs);
        return list;
    }

    public NotificationsTO getNotificationsTO(Long currentid, Notifications n) {
        String text = n.getDescription();
        String date = DateUtils.DateDecorator(n.getUpdatedAt());
        Long idFrom = getIdFrom(n);
        UserTO uTo = userBuilder.createUserTOSimple(n.getUser_to(),currentid);
        GrupTO gTo = grupBuilder.createTOSimple(n.getGrup_to(),currentid,null,null);
        UserTO uFrom = userBuilder.createUserTOSimple(n.getUser_from(),currentid);
        GrupTO gFrom = grupBuilder.createTOSimple(n.getGrup_from(),currentid,null,null);

        return new NotificationsTO(ModelJSON.OK,ModelJSON.MSG_EMPTY, currentid,
                n.getId(),text, n.getIsNew(), n.getStatus(),date,n.getTypeOf(),idFrom,uTo,gTo,uFrom,gFrom);
    }


    private String templateTextNotificationToHTML(Notifications notification){

        if(notification.getTypeOf().equals(TypeNotifications.SYSTEM_EDIT_PROFILE)){
            return "<strong>SocialSong</strong> " + notification.getDescription();

        }else if(notification.getTypeOf().equals(TypeNotifications.FOLLOW)){
            return "<strong>"  + StringUtils.ConvertirPrimeraLetraEnMayuscula(notification.getUser_from().toString())+
                    "</strong>"  + " t'està seguint.";

        }else if(notification.getTypeOf().equals(TypeNotifications.FOLLOW_GRUP)){
            return "<strong>"  +StringUtils.ConvertirPrimeraLetraEnMayuscula(notification.getUser_from().toString())  +
                    "</strong> està seguint a <strong>" + StringUtils.ConvertirPrimeraLetraEnMayuscula(notification.getGrup_to().getNom())
                    + "</strong>.";

        }else if(notification.getTypeOf().equals(TypeNotifications.FRIEND_POST)
                || notification.getTypeOf().equals(TypeNotifications.NO_FRIEND_POST)){
            return "<strong>" + StringUtils.ConvertirPrimeraLetraEnMayuscula(notification.getUser_from().toString())  +
                    "</strong> a fet una publicació en el teu perfil.";

        }else if(notification.getTypeOf().equals(TypeNotifications.GRUP_POST)){
            return "<strong>" +StringUtils.ConvertirPrimeraLetraEnMayuscula(notification.getGrup_to().getNom())  +
                    "</strong> a fet una publicació en el teu perfil.";
        }

        return null;
    }

    private Long getIdFrom(Notifications notification){

        if(notification.getTypeOf().equals(TypeNotifications.SYSTEM_EDIT_PROFILE)){
            return null;

        }else if(notification.getTypeOf().equals(TypeNotifications.FOLLOW) ||
                notification.getTypeOf().equals(TypeNotifications.FOLLOW_GRUP)){
            return notification.getUser_from().getId();


        }else if(notification.getTypeOf().equals(TypeNotifications.FRIEND_POST)
                || notification.getTypeOf().equals(TypeNotifications.NO_FRIEND_POST)){
            return notification.getUser_to().getId();

        }else if(notification.getTypeOf().equals(TypeNotifications.GRUP_POST)){
            return notification.getGrup_from().getId();

        }

        return null;
    }

    public void notificationResponse(Notifications notifications, Boolean option, Long id) {

        if(TypeNotifications.GRUP_MEMBER.equals(notifications.getTypeOf())){

            GrupMember grupMember = grup_userService.match(notifications.getGrup_from().getId(),
                    notifications.getUser_to().getId());

            if(option){
                grupMember.setStatus(Status.OK);

                grupBuilder.doFollowGrup(notifications.getUser_to(),notifications.getGrup_from());

            }else{
                grupMember.setStatus(Status.NO);
            }
            grup_userService.update(grupMember);
        }
    }
}
