package com.tfg.api.projecte_tfg.rest;

import com.tfg.api.projecte_tfg.builder.AuthBuilder;
import com.tfg.api.projecte_tfg.enums.Status;
import com.tfg.api.projecte_tfg.enums.TypeNotifications;
import com.tfg.api.projecte_tfg.model.Auth;
import com.tfg.api.projecte_tfg.model.Grup;
import com.tfg.api.projecte_tfg.model.Notifications;
import com.tfg.api.projecte_tfg.model.User;
import com.tfg.api.projecte_tfg.rest.object.ModelJSON;
import com.tfg.api.projecte_tfg.service.AuthService;
import com.tfg.api.projecte_tfg.service.InfoUserService;
import com.tfg.api.projecte_tfg.service.NotificationsService;
import com.tfg.api.projecte_tfg.service.UserService;

import javax.ejb.EJB;
import javax.enterprise.context.RequestScoped;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.ws.rs.*;
import javax.ws.rs.core.Context;


/**
 * Created by marc on 18/03/14.
 */

// This class is used to process all the authentication related URLs
@Path("/auth")
@RequestScoped
public class AuthRESTService {
    // This is the EJB used to access user data
    @EJB
    AuthService authService;
    @EJB
    UserService userService;
    @EJB
    NotificationsService notificationsService;
    @EJB
    private InfoUserService ius;
    @EJB
    AuthBuilder ab;

    private static String CAT_MSG = "et recomana completar el teu perfil.";

    @Path("/init")
    @POST
    @Produces("application/json")
    public ModelJSON getInit(
            @Context HttpServletRequest req) {

        // Access to the HTTP session
        HttpSession session = req.getSession();

        if (session == null) {
            return error("Error: Error de session");
        }

        if (session.getAttribute("web_auth_id") == null) {
            return error();
        }
        else {

            Long id = (Long) session.getAttribute("web_auth_id");
            Auth au = authService.findbyUserId(id);

            if (au != null){
                return createOK(au.getUser().getId());
            }else{

                session.setAttribute("web_auth_id", null);
                return error();
            }
        }
    }

    private ModelJSON error(){
        return new ModelJSON(ModelJSON.ERROR, ModelJSON.MSG_EMPTY,null);

    }

    private ModelJSON error(String s){
        return new ModelJSON(ModelJSON.ERROR, s,null);

    }

    private ModelJSON createOK(Long currentId){
        return new ModelJSON(ModelJSON.OK, ModelJSON.MSG_EMPTY,currentId);

    }
    private ModelJSON createOK(Long currentId,String s){
        return new ModelJSON(ModelJSON.OK, s,currentId);

    }


    @POST
    @Produces("application/json")
    public ModelJSON auth(
            @Context HttpServletRequest req,
            @FormParam("email") String mail,
            @FormParam("pwd") String password) {

        // Access to the HTTP session
        HttpSession session = req.getSession();

        if (session == null) {
            return error("Error: Error de session");
        }

        if (session.getAttribute("web_auth_id") == null) {

            Auth au = authService.matchPassword(mail, password);
            if (au != null) {
                try {
                    session.setAttribute("web_auth_id", au.getUser().getId());
                    return createOK(au.getId());
                }
                catch (Exception ex) {
                    return error("Error: Error del Servidor");

                }
            }
            else
            return error("Error: email o contraseña incorrecta");

        }
        else {
            return createOK((Long) session.getAttribute("web_auth_id"),"Error: Sessio iniciada");
        }
    }

    @Path("/register")
    @POST
    @Produces("application/json")
    public ModelJSON register(
            @Context HttpServletRequest req,
            @FormParam("name") String username,
            @FormParam("secondname") String secondname,
            @FormParam("email") String email,
            @FormParam("password") String password,
            @FormParam("password2") String password2){

        HttpSession session = req.getSession();

        if (session == null) {
            return error("Error: Error de session");

        }

        if (session.getAttribute("web_auth_id") != null) {
            return error("Error: Error de session");

        }

        Auth au = authService.findbyEmail(email);

        if(au != null){
            return error("Error: email ya registrado");
        }

        au = authService.save(new Auth(email, password));

        if(au != null){

            if(username != null){
                username = username.trim();
                username = username.replace("  "," ");
            }

            if(secondname != null){
                secondname = secondname.trim();
                secondname = secondname.replace("  "," ");

            }

            User u = userService.save(new User(username, secondname));
            if(u != null){

                Notifications notifications = notificationsService.save(new Notifications(
                        TypeNotifications.SYSTEM_EDIT_PROFILE, true, Status.OK,
                        CAT_MSG, u,null, null, null));

                au.setUser(u);
                au = authService.merge(au);
                ius.save(ab.defauldInfoUser(u,au));
                session.setAttribute("web_auth_id", u.getId());
                return createOK(u.getId());
            }

        }else{
            return error("Error: Error del registro");

        }

        return error("Error: Error del registro");
    }

    @Path("/logout")
    @POST
    @Produces("application/json")
    public ModelJSON logout(
            @Context HttpServletRequest req){

        HttpSession session = req.getSession();

        if (session == null) {
            return error("Error: Error de session");

        }

        if (session.getAttribute("web_auth_id") != null) {
            session.setAttribute("web_auth_id", null);
            return new ModelJSON(ModelJSON.OK, "",null);

        }

        return createOK(null,"No session.");
    }
}