package com.tfg.api.projecte_tfg.rest;

import com.tfg.api.projecte_tfg.builder.EventGrupBuilder;
import com.tfg.api.projecte_tfg.enums.Member;
import com.tfg.api.projecte_tfg.enums.Status;
import com.tfg.api.projecte_tfg.enums.TypeNotifications;
import com.tfg.api.projecte_tfg.model.*;
import com.tfg.api.projecte_tfg.rest.object.EventTO;
import com.tfg.api.projecte_tfg.rest.object.FriendTO;
import com.tfg.api.projecte_tfg.rest.object.ListTO;
import com.tfg.api.projecte_tfg.rest.object.ModelJSON;
import com.tfg.api.projecte_tfg.service.*;
import com.tfg.api.projecte_tfg.util.DateUtils;
import com.tfg.api.projecte_tfg.util.StringUtils;

import javax.ejb.EJB;
import javax.enterprise.context.RequestScoped;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.ws.rs.*;
import javax.ws.rs.core.Context;
import java.util.Date;

/**
 * Created by marc on 08/05/14.
 */
@Path("/event")
@RequestScoped
public class EventGrupRESTService {

    @EJB
    private EventGrupService eventGrupService;
    @EJB
    private PlaceService placeService;
    @EJB
    private GrupService grupService;
    @EJB
    private EventGrupBuilder eventGrupBuilder;
    @EJB
    private Grup_UserService grup_userService;
    @EJB
    private UserService userService;

    @POST
    @Produces("application/json")
    public EventTO createEvent(@Context HttpServletRequest req,
                                  @FormParam("groupId") Long groupId,
                                  @FormParam("place") String place,
                                  @FormParam("description") String description,
                                  @FormParam("date") String date) throws Exception {

        // Access to the HTTP session
        HttpSession session = req.getSession();

        if (session == null) {
            return eventGrupBuilder.createError("Error: Error de session");
        }

        if (session.getAttribute("web_auth_id") == null) {
            return eventGrupBuilder.createError(ModelJSON.MSG_EMPTY);
        }

        Long currentId = getSessionUserId(session);

        Grup grup = grupService.getGrup(groupId);
        User user = userService.getUser(currentId);

        if(grup == null || user == null){return eventGrupBuilder.createError("Error en la cerca del grup i/o usuari");}

        if(!isMemberAdminOrFounder(grup,user)){return eventGrupBuilder.createError("l'usuari no és membre administrador o fundador");}

        try{

            Place placeEvent = checkPlace(place);

            Date dateEvent = DateUtils.createWithHour(date);

            EventGrup eventGrup = createAndSave(description, grup, placeEvent, dateEvent);

            return eventGrupBuilder.create(eventGrup,currentId);

        }   catch (Exception e){

            return eventGrupBuilder.createError("Error al crear l'esdeveniment");
        }

    }

    private boolean isMemberAdminOrFounder(Grup grup, User user) {
        GrupMember grupMember = grup_userService.match(grup.getId(), user.getId());

        return Status.OK.equals(grupMember.getStatus())
                && (Member.FOUNDER.equals(grupMember.getTypemember())
                || Member.ADMIN.equals(grupMember.getTypemember()));
    }

    private EventGrup createAndSave(String description, Grup grup, Place placeEvent, Date dateEvent) {
        EventGrup eventGrup = new EventGrup(grup,placeEvent,description,dateEvent);

        eventGrupService.save(eventGrup);
        return eventGrup;
    }

    private Place checkPlace(String place) {
        Place placeEvent = null;
        if(StringUtils.notEmptyChar(place)){
            place = place.trim();
            placeEvent = placeService.getByName(place);

            if(placeEvent == null){
                placeEvent = new Place(place,"","");
                placeService.save(placeEvent);
            }
        }
        return placeEvent;
    }

    private Long getSessionUserId(HttpSession session) {
        return (Long) session.getAttribute("web_auth_id");
    }
}
