package com.tfg.api.projecte_tfg.rest;

import com.tfg.api.projecte_tfg.builder.FriendBuilder;
import com.tfg.api.projecte_tfg.builder.UserBuilder;
import com.tfg.api.projecte_tfg.enums.Status;
import com.tfg.api.projecte_tfg.enums.TypeNotifications;
import com.tfg.api.projecte_tfg.model.Friend;
import com.tfg.api.projecte_tfg.model.Notifications;
import com.tfg.api.projecte_tfg.model.User;
import com.tfg.api.projecte_tfg.rest.object.FriendTO;
import com.tfg.api.projecte_tfg.rest.object.ListTO;
import com.tfg.api.projecte_tfg.rest.object.ModelJSON;
import com.tfg.api.projecte_tfg.rest.object.UserTO;
import com.tfg.api.projecte_tfg.service.FriendService;
import com.tfg.api.projecte_tfg.service.NotificationsService;

import javax.ejb.EJB;
import javax.enterprise.context.RequestScoped;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.ws.rs.*;
import javax.ws.rs.core.Context;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by marc on 02/04/14.
 */

@Path("/friend")
@RequestScoped
public class FriendRESTService {

    @EJB
    private FriendService fs;
    @EJB
    private FriendBuilder fb;

    @EJB
    private UserBuilder userBuilder;
    @EJB
    private NotificationsService notificationsService;

    @GET
    @Path("/{id}")
    @Produces("application/json")
    public FriendTO getInfoRelationship(@Context HttpServletRequest req,
                            @PathParam("id") Long idUser) throws Exception {

        // Access to the HTTP session
        HttpSession session = req.getSession();

        if (session == null) {
            return new FriendTO(ModelJSON.ERROR, "Error: Error de session",null,null,null,null);
        }

        if (session.getAttribute("web_auth_id") == null) {
            return new FriendTO(ModelJSON.ERROR, ModelJSON.MSG_EMPTY,null,null,null,null);
        }else if (idUser != null){

            Long id = (Long) session.getAttribute("web_auth_id");
            Friend f = fs.match(id, idUser);
            return fb.createObject(f,id,id,idUser);

        }else{
            return new FriendTO(ModelJSON.ERROR, "Error: id null",null,null,null,null);
        }

    }

    @GET
    @Path("/followers/{id}")
    @Produces("application/json")
    public ListTO<UserTO> getFollowers(@Context HttpServletRequest req,
                                    @PathParam("id") Long userId) throws Exception {

        // Access to the HTTP session
        HttpSession session = req.getSession();

        if (session == null) {
            return new ListTO(ModelJSON.ERROR, "Error: Error de session",null,null);
        }

        if (session.getAttribute("web_auth_id") == null) {
            return new ListTO(ModelJSON.ERROR, ModelJSON.MSG_EMPTY,null,null);
        }else if (userId != null){

            Long currentId = (Long) session.getAttribute("web_auth_id");

            List<Friend> allFollowers = fs.getAllFollowers(userId);

            List<User> userList = new ArrayList<User>(allFollowers.size());

            for(Friend f : allFollowers){
                userList.add(f.getUser1());
            }

            List<UserTO> listUserTO = userBuilder.ListTO(userList, currentId);
            ListTO<UserTO> userTOListTO = new ListTO<UserTO>(currentId);
            userTOListTO.setList(listUserTO);

            return userTOListTO;

        }else{
            return new ListTO(ModelJSON.ERROR, ModelJSON.MSG_EMPTY,null,null);
        }
    }

    @GET
    @Path("/follows/{id}")
    @Produces("application/json")
    public ListTO<UserTO> getFollows(@Context HttpServletRequest req,
                                    @PathParam("id") Long userId) throws Exception {

        // Access to the HTTP session
        HttpSession session = req.getSession();

        if (session == null) {
            return new ListTO(ModelJSON.ERROR, "Error: Error de session",null,null);
        }

        if (session.getAttribute("web_auth_id") == null) {
            return new ListTO(ModelJSON.ERROR, ModelJSON.MSG_EMPTY,null,null);
        }else if (userId != null){

            Long currentId = (Long) session.getAttribute("web_auth_id");

            List<Friend> allFollows = fs.getAllFollows(userId);

            List<User> userList = new ArrayList<User>(allFollows.size());

            for(Friend f : allFollows){
                userList.add(f.getUser2());
            }

            List<UserTO> listUserTO = userBuilder.ListTO(userList, currentId);
            ListTO<UserTO> userTOListTO = new ListTO<UserTO>(currentId);
            userTOListTO.setList(listUserTO);

            return userTOListTO;

        }else{
            return new ListTO(ModelJSON.ERROR, ModelJSON.MSG_EMPTY,null,null);
        }
    }

    @POST
    @Produces("application/json")
    public FriendTO createFriends(@Context HttpServletRequest req,
                                  @FormParam("id") Long idUser) throws Exception {

        // Access to the HTTP session
        HttpSession session = req.getSession();

        if (session == null) {
            return new FriendTO(ModelJSON.ERROR, "Error: Error de session",null,null,null,null);
        }

        if (session.getAttribute("web_auth_id") == null) {
            return new FriendTO(ModelJSON.ERROR, ModelJSON.MSG_EMPTY,null,null,null,null);
        }else if (idUser != null){

            Long id = (Long) session.getAttribute("web_auth_id");
            Friend f = fs.match(id,idUser);

            if(f == null){
                    f = fs.save(fb.crearFriend(id,idUser));
            }else{
                f.setStatus(Status.OK);
                f = fs.update(f);
            }

            if(f != null){

                Notifications notifications = notificationsService.save(new Notifications(
                        TypeNotifications.FOLLOW, true, Status.OK,
                        " t'està seguint.", f.getUser2(),null, f.getUser1(), null));

                return fb.createObject(f,id,id,idUser);
            }else{
                return new FriendTO(ModelJSON.ERROR, "Error: Error en Seguir",null,null,null,null);

            }

        }else{
            return new FriendTO(ModelJSON.ERROR, "Error: id null",null,null,null,null);
        }

    }

    @PUT
    @Path("/update")
    @Produces("application/json")
    public FriendTO modifyFriends(@Context HttpServletRequest req,
                                  @FormParam("id") Long idUser,
                                  @FormParam("friend") Boolean friend) throws Exception {

        // Access to the HTTP session
        HttpSession session = req.getSession();

        if (session == null) {
            return new FriendTO(ModelJSON.ERROR, "Error: Error de session",null,null,null,null);
        }

        if (session.getAttribute("web_auth_id") == null) {
            return new FriendTO(ModelJSON.ERROR, ModelJSON.MSG_EMPTY,null,null,null,null);
        }else if (idUser != null){

            Long id = (Long) session.getAttribute("web_auth_id");
            Friend f = fs.match(id,idUser);

            if(friend){
                f.setStatus(Status.OK);
                f = fs.update(f);

                Notifications notifications = notificationsService.save(new Notifications(
                        TypeNotifications.FOLLOW, true, Status.OK,
                        " t'està seguint.", f.getUser2(),null, f.getUser1(), null));
            }else{
                f.setStatus(Status.NO);
                f = fs.update(f);
            }



            if(f != null){
                return fb.createObject(f,id,id,idUser);
            }else{
                return new FriendTO(ModelJSON.ERROR, "Error: Error en Seguir",null,null,null,null);

            }

        }else{
            return new FriendTO(ModelJSON.ERROR, "Error: id null",null,null,null,null);
        }

    }


}
