package com.tfg.api.projecte_tfg.rest;

import com.tfg.api.projecte_tfg.builder.EventGrupBuilder;
import com.tfg.api.projecte_tfg.builder.GrupBuilder;
import com.tfg.api.projecte_tfg.enums.Member;
import com.tfg.api.projecte_tfg.enums.Status;
import com.tfg.api.projecte_tfg.enums.TypeNotifications;
import com.tfg.api.projecte_tfg.model.*;
import com.tfg.api.projecte_tfg.rest.object.EventTO;
import com.tfg.api.projecte_tfg.rest.object.GrupTO;
import com.tfg.api.projecte_tfg.rest.object.ListTO;
import com.tfg.api.projecte_tfg.rest.object.ModelJSON;
import com.tfg.api.projecte_tfg.service.*;
import com.tfg.api.projecte_tfg.util.DateUtils;
import com.tfg.api.projecte_tfg.util.StringUtils;

import javax.ejb.EJB;
import javax.enterprise.context.RequestScoped;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.ws.rs.*;
import javax.ws.rs.core.Context;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Created by marc on 22/04/14.
 */

@Path("/grup")
@RequestScoped
public class GrupRESTService {

    @EJB
    private GrupService grupService;
    @EJB
    private GrupBuilder grupBuilder;
    @EJB
    private StyleMusicService styleMusicService;
    @EJB
    private UserService userService;
    @EJB
    private Grup_UserService grup_userService;
    @EJB
    private FollowGrupService followGrupService;
    @EJB
    private FriendService friendService;
    @EJB
    private EventGrupService eventGrupService;
    @EJB
    private EventGrupBuilder eventGrupBuilder;
    @EJB
    NotificationsService notificationsService;
    @EJB
    ImageUserService imageUserService;

    @POST
    @Produces("application/json")
    public GrupTO createGroup(
            @Context HttpServletRequest req,
            @FormParam("name") String name,
            @FormParam("description") String description,
            @FormParam("founded") String founded,
            @FormParam("photo") String photo,
            @FormParam("members") String members,
            @FormParam("styleMusica") String styleMusica) throws Exception {

        HttpSession session = req.getSession();

        if (isSessionError(session)) {

            return grupBuilder.error("Error: Error de session",null);
        }

        if (noAuth(session)) {

            return grupBuilder.error("Error: Sessio iniciada",null);
        }

        Long id = getSessionUserId(session);
        User user = userService.getUser(id);

        if(user == null){
            return grupBuilder.error("Error: user null",null);
        }

        Grup grup = grupService.find(name);

        if(grup == null){

            StyleMusic s = checkMusicStyle(styleMusica);

            try {

                ImageUser imageUser = checkImage(photo);

                grup = grupBuilder.create(name,description,founded,s,imageUser);
                grup = grupService.saveAndUpdate(grup);

            } catch (ParseException e) {
                return grupBuilder.error("Error: Error al crear el grup",id);
            }

            grup_userService.save(new GrupMember(user, grup, Status.OK, Member.FOUNDER,""));
            followGroup(req, grup.getId(), true);

            createNotificationCrearGrup(Status.OK, "Has creat el grup ", user, grup);

            createMembersGroup(members, grup);

            return createGroupTO(id, grup);
        }else{

            return grupBuilder.error("Error: El Nom del Grup ya existeix",id);
        }

    }

    private StyleMusic checkMusicStyle(String styleMusica) {
        StyleMusic s = styleMusicService.find(styleMusica);

        if(s == null){
            s = styleMusicService.saveAndUpdate(new StyleMusic(styleMusica,styleMusica));
        }
        return s;
    }

    private GrupTO createGroupTO(Long id, Grup grup) {
        List<GrupMember> grupMemberList = grup_userService.getUsersByGrupId(grup.getId());
        Long grupId = grup.getId();

        List<EventTO> eventTOList = makeListEventTO(id, grupId);

        return grupBuilder.createTO(grup, id,grupMemberList,eventTOList);
    }

    private void createMembersGroup(String members, Grup grup) {
        List<Long> lMembers = StringUtils.makeLongArray(members);

        List<User> lu = null;
        if(lMembers != null && !lMembers.isEmpty())   {
            lu = new ArrayList<User>(lMembers.size());

            for(Long _id : lMembers){
                try{
                    lu.add(userService.getUser(_id));
                }catch (Exception e){}
            }
        }

        List<GrupMember> grupMembers = grupBuilder.makeListMembersBasic(lu, grup);

        if(grupMembers != null && !grupMembers.isEmpty()){
            for(GrupMember grupUser : grupMembers){
                GrupMember grup_member = grup_userService.save(grupUser);
                createNotificationInvitation(grupUser.getGrup(), grupUser.getUser());
            }
        }
    }

    private ImageUser checkImage(String photo) {
        ImageUser imageUser = null;
        try{
            if(photo != null){

                String[] img = photo.split("base64,");
                String image = img[1];

                byte[] decoded = org.apache.commons.codec.binary.Base64.decodeBase64(image.getBytes());

                imageUser = new ImageUser(null, decoded,0f,"","Principal");
                imageUserService.save(imageUser);
            }
            return imageUser;

        }catch (Exception e){
            return null;
        }
    }

    private Long getSessionUserId(HttpSession session) {
        return (Long) session.getAttribute("web_auth_id");
    }

    private boolean noAuth(HttpSession session) {
        return session.getAttribute("web_auth_id") == null;
    }

    private boolean isSessionError(HttpSession session) {
        return session == null;
    }

    private List<EventTO> makeListEventTO(Long id, Long grupId) {
        List<EventGrup> eventGrupList = eventGrupService.getActiveEventsTop10ByGrupId(grupId);
        List<EventTO> eventTOList = null;
        if(eventGrupList != null){
            eventTOList = new ArrayList<EventTO>(eventGrupList.size());

            for (EventGrup e : eventGrupList){
                eventTOList.add(eventGrupBuilder.create(e,id));
            }
        }
        return eventTOList;
    }

    @PUT
    @Path("{id}")
    @Produces("application/json")
    public GrupTO editGroup(
            @Context HttpServletRequest req,
            @PathParam("id") Long grupId,
            @FormParam("name") String name,
            @FormParam("description") String description,
            @FormParam("founded") String founded,
            @FormParam("photo") String photo,
            @FormParam("styleMusica") String styleMusica) throws Exception {

        HttpSession session = req.getSession();

        if (isSessionError(session)) {

            return grupBuilder.error("Error: Error de session",null);
        }

        if (noAuth(session)) {

            return grupBuilder.error("Error: Sessio iniciada",null);
        }

        Long id = getSessionUserId(session);
        User user = userService.getUser(id);
        if(user == null){
            return grupBuilder.error("Error: user null",null);
        }

        Grup grup = grupService.getGrup(grupId);

        if(grup == null){

            return grupBuilder.error("Error: No es troba el grup.",id);

        }else{

            if(StringUtils.notEmptyChar(name)){
                grup.setNom(name);
            }

            if(description != null){
                grup.setDescription(description);
            }

            if(StringUtils.notEmptyChar(founded)){
                Date f = DateUtils.create(founded);
                grup.setFounded(f);
            }

            if(photo != null){

                String[] img = photo.split("base64,");
                String image = img[1];

                byte[] decoded = org.apache.commons.codec.binary.Base64.decodeBase64(image.getBytes());

                ImageUser imageUser = new ImageUser(null, decoded,0f,"","Principal");
                imageUserService.save(imageUser);

                grup.setImage(imageUser);
            }

            if(StringUtils.notEmptyChar(styleMusica)){
                StyleMusic s = checkMusicStyle(styleMusica);
                grup.setStyleMusic(s);
            }

            grupService.update(grup);

            return createGroupTO(grupId, id, grup);
        }

    }


    @PUT
    @Path("member/{id}")
    @Produces("application/json")
    public GrupTO editMember(
            @Context HttpServletRequest req,
            @PathParam("id") Long grupId,
            @FormParam("memberId") Long memberId,
            @FormParam("type") String type,
            @FormParam("rol") String rol) throws Exception {

        HttpSession session = req.getSession();

        if (isSessionError(session)) {

            return grupBuilder.error("Error: Error de session",null);
        }

        if (noAuth(session)) {

            return grupBuilder.error("Error: Sessio iniciada",null);
        }

        Long id = getSessionUserId(session);

        Grup grup = grupService.getGrup(grupId);

        if(grup == null){

            return grupBuilder.error("Error: No es troba el grup.",id);

        }else{

            try {

                GrupMember grupMember = grup_userService.match(grupId,memberId);

                User user = grupMember.getUser();

                Member member = Member.BASIC;

                if(Member.ADMIN.toString().equals(type)){
                    member = Member.ADMIN;
                }

                if(grupMember == null){

                    grupMember = grupBuilder.createGrupMember(grup,user,rol,member);
                    grup_userService.save(grupMember);
                    createNotificationInvitation(grup, user);

                }else{
                    if(grupMember.getStatus().equals(Status.NO)){
                        grupMember.setStatus(Status.PENDING);
                        createNotificationInvitation(grup, user);
                    }
                    if(! Member.FOUNDER.equals(grupMember.getTypemember())){
                        grupMember.setTypemember(member);
                    }

                    grupMember.setRol(rol);
                    grup_userService.update(grupMember);
                }


                return createGroupTO(grupId, id, grup);

            }catch (Exception e){
                return grupBuilder.error("Error: error inesperat.",id);

            }

        }
    }

    private void createNotificationInvitation(Grup grup, User user) {
        createNotification(Status.PENDING, "Tens una invitació per ser membre", user, grup);
    }

    @POST
    @Path("member/{id}")
    @Produces("application/json")
    public GrupTO createMember(
            @Context HttpServletRequest req,
            @PathParam("id") Long grupId,
            @FormParam("userId") Long userId,
            @FormParam("type") String type,
            @FormParam("rol") String rol) throws Exception {

        HttpSession session = req.getSession();

        if (isSessionError(session)) {

            return grupBuilder.error("Error: Error de session",null);
        }

        if (noAuth(session)) {

            return grupBuilder.error("Error: Sessio iniciada",null);
        }

        Long id = getSessionUserId(session);

        Grup grup = grupService.getGrup(grupId);

        if(grup == null){

            return grupBuilder.error("Error: No es troba el grup.",id);

        }else{

            try {

                User user = userService.getUser(userId);

                GrupMember grupMember = grup_userService.match(grupId,user.getId());

                Member member = Member.BASIC;

                if(Member.ADMIN.toString().equals(type)){
                    member = Member.ADMIN;
                }

                if(grupMember == null){

                    grupMember = grupBuilder.createGrupMember(grup,user,rol,member);
                    grup_userService.save(grupMember);
                    createNotificationInvitation(grup, user);

                }else{
                    if(grupMember.getStatus().equals(Status.NO)){
                        crearInvitacioMembre(grup, user, grupMember);
                    }
                    if(! Member.FOUNDER.equals(grupMember.getTypemember())){
                        grupMember.setTypemember(member);
                    }

                    grupMember.setRol(rol);
                    grup_userService.update(grupMember);
                }


                return createGroupTO(grupId, id, grup);

            }catch (Exception e){
                return grupBuilder.error("Error: error inesperat.",id);

            }

        }
    }

    private GrupTO createGroupTO(Long grupId, Long id, Grup grup) {
        List<GrupMember> members = grup_userService.getUsersByGrupId(grupId);
        List<EventTO> eventTOList = makeListEventTO(id, grupId);
        return grupBuilder.createTO(grup,id,members,eventTOList);
    }

    private void crearInvitacioMembre(Grup grup, User user, GrupMember grupMember) {
        grupMember.setStatus(Status.PENDING);
        createNotificationInvitation(grup, user);
    }

    @DELETE
    @Path("member/{id}/{memberId}")
    @Produces("application/json")
    public GrupTO deleteMember(
            @Context HttpServletRequest req,
            @PathParam("id") Long grupId,
            @PathParam("memberId") Long memberId) throws Exception {

        HttpSession session = req.getSession();

        if (isSessionError(session)) {

            return grupBuilder.error("Error: Error de session",null);
        }

        if (noAuth(session)) {

            return grupBuilder.error("Error: Sessio iniciada",null);
        }

        Long id = getSessionUserId(session);

        Grup grup = grupService.getGrup(grupId);

        if(grup == null){

            return grupBuilder.error("Error: No es troba el grup.",id);

        }else{

            try{

                GrupMember grupMember = grup_userService.get(memberId);

                User user = grupMember.getUser();

                if(grupMember != null){
                    grupMember.setStatus(Status.NO);
                    grup_userService.update(grupMember);
                    createNotification(Status.OK,"Has deixat de ser membre",user,grup);
                }

                grup_userService.save(grupMember);

                return createGroupTO(grupId, id, grup);

            }catch (Exception e){
                return grupBuilder.error("Error: error inesperat.",id);

            }

        }
    }

    @GET
    @Path("{id}")
    @Produces("application/json")
    public GrupTO getInfoGroup(@Context HttpServletRequest req,
                              @PathParam("id") Long grupId) throws Exception {

        // Access to the HTTP session
        HttpSession session = req.getSession();

        if (isSessionError(session)) {
            return grupBuilder.error("Error: Error de session",null);

        }

        if (noAuth(session)) {
            return grupBuilder.error(ModelJSON.MSG_EMPTY,null);
        }


        Long id = getSessionUserId(session);
        User user = userService.getUser(id);
        if(user == null){
            return grupBuilder.error("Error: user null",null);
        }

        Grup g = grupService.getGrup(grupId);
        List<GrupMember> l = grup_userService.getUsersByGrupId(g.getId());
        List<EventTO> eventTOList = makeListEventTO(id, grupId);

        return grupBuilder.createTO(g,id,l,eventTOList);

    }

    @GET
    @Path("favourites/{id}")
    @Produces("application/json")
    public ListTO<GrupTO> getFavourites(@Context HttpServletRequest req,
                              @PathParam("id") Long userId) throws Exception {

        // Access to the HTTP session
        HttpSession session = req.getSession();

        if (isSessionError(session)) {
            return new ListTO<GrupTO>(ModelJSON.ERROR,"Error: Error de session",null,null);

        }

        if (noAuth(session)) {
            return new ListTO<GrupTO>(ModelJSON.ERROR,"No auth",null,null);
        }


        Long id = getSessionUserId(session);
        User user = userService.getUser(id);
        if(user == null){
            return new ListTO<GrupTO>(ModelJSON.ERROR,"Error: user null",null,null);
        }

        List<Grup> list = followGrupService.getListGrupsFollow(userId);

        return grupBuilder.ListGrupTOLite(list, id);

    }

    @PUT
    @Path("/follow")
    @Produces("application/json")
    public GrupTO followGroup(@Context HttpServletRequest req,
                                  @FormParam("grupId") Long grupId,
                                  @FormParam("follow") Boolean follow) throws Exception {

        // Access to the HTTP session
        HttpSession session = req.getSession();

        if (isSessionError(session)) {
            return grupBuilder.error("Error: Error de session",null);

        }

        if (noAuth(session)) {
            return grupBuilder.error(ModelJSON.MSG_EMPTY,null);

        }else if (grupId != null){

            follow = follow == null ? false : follow;
            Long id = getSessionUserId(session);

            FollowGrup followGrup = followGrupService.match(id,grupId);
            Grup grup = grupService.getGrup(grupId);

            if(followGrup == null){

                createFollow(follow, id, grup);

            }else{

                modifyFollow(follow, followGrup);
            }

            return createGroupTO(id, grup);

        }else{
            return grupBuilder.error(ModelJSON.MSG_EMPTY,null);
        }

    }

    private void modifyFollow(Boolean follow, FollowGrup followGrup) {
        followGrup.setFollow(follow);
        followGrupService.update(followGrup);
    }

    private void createFollow(Boolean follow, Long id, Grup grup) throws Exception {
        FollowGrup followGrup;User user = userService.getUser(id);
        followGrup = new FollowGrup(grup, user, follow);
        followGrupService.save(followGrup);
    }

    private void createNotification(Status s, String msg,User u,Grup g){
        Notifications notifications = notificationsService.save(new Notifications(
                TypeNotifications.GRUP_MEMBER, true, s,
                msg, u,null, null, g));
    }

    private void createNotificationCrearGrup(Status s, String msg,User u,Grup g){
        Notifications notifications = notificationsService.save(new Notifications(
                TypeNotifications.GRUP, true, s,
                msg, u,null, null, g));
    }

}
