package com.tfg.api.projecte_tfg.rest;

import com.tfg.api.projecte_tfg.builder.PostBuilder;
import com.tfg.api.projecte_tfg.model.*;
import com.tfg.api.projecte_tfg.rest.object.ModelJSON;
import com.tfg.api.projecte_tfg.rest.object.PostTO;
import com.tfg.api.projecte_tfg.service.CommentaryService;
import com.tfg.api.projecte_tfg.service.LikePostService;
import com.tfg.api.projecte_tfg.service.PostService;
import com.tfg.api.projecte_tfg.service.UserService;

import javax.ejb.EJB;
import javax.enterprise.context.RequestScoped;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.ws.rs.*;
import javax.ws.rs.core.Context;

/**
 * Created by marc on 13/05/14.
 */
@Path("/like")
@RequestScoped
public class LikePostRESTService {

    @EJB
    private PostBuilder pb;
    @EJB
    private LikePostService likePostService;
    @EJB
    private PostService postService;
    @EJB
    private UserService userService;
    @EJB
    private CommentaryService commentaryService;

    private boolean isSessionError(HttpSession session) {
        if (session == null) {
            return true;
        }
        return false;
    }


    @POST
    @Produces("application/json")
    public PostTO createLike(@Context HttpServletRequest req,
                            @FormParam("postId") Long postId,
                            @FormParam("commentaryId") Long commentaryId) throws Exception {

        // Access to the HTTP session
        HttpSession session = req.getSession();

        if (isSessionError(session)) return pb.createError("Error: Error de session");

        if (session.getAttribute("web_auth_id") == null) {
            return pb.createError(ModelJSON.MSG_EMPTY);
        } else {

            Long id = (Long) session.getAttribute("web_auth_id");
            User user = userService.getUser(id);



            if (user == null){
                return pb.createError(ModelJSON.MSG_EMPTY);
            }else{

                LikePost likePost = createAndSave( postId,commentaryId, user);

                Post p = null;

                if(likePost != null){
                    p = likePost.getPost();
                }

                if(p == null && likePost != null && likePost.getCommentary() != null){
                    p = likePost.getCommentary().getPost();
                }
                return pb.createPostTO(p,id);
            }
        }
    }

    private LikePost createAndSave(Long postId, Long commentaryId, User user){
        Post post = null;
        Commentary commentary = null;

        if(postId != null && postId != -1){
            post = postService.find(postId);
        }else if(commentaryId != null && commentaryId != -1){
            commentary = commentaryService.find(commentaryId);
        }

        LikePost likePost = new LikePost(user,post,commentary,true);
        likePostService.save(likePost);
        return likePost;
    }


    @PUT
    @Produces("application/json")
    public PostTO modifyLike(@Context HttpServletRequest req,
                            @FormParam("postId") Long postId,
                            @FormParam("commentaryId") Long commentaryId,
                            @FormParam("like") Boolean like) throws Exception {

        // Access to the HTTP session
        HttpSession session = req.getSession();

        if (isSessionError(session)) return pb.createError("Error: Error de session");

        if (session.getAttribute("web_auth_id") == null) {
            return pb.createError(ModelJSON.MSG_EMPTY);
        } else {

            Long id = (Long) session.getAttribute("web_auth_id");
            User user = userService.getUser(id);

            if (user == null){
                return pb.createError(ModelJSON.MSG_EMPTY);
            }else{

                LikePost likePost = null;

                if(postId != null && postId != -1){
                    likePost = likePostService.matchPost(postId, user.getId());
                }else if(commentaryId != null && commentaryId != -1){
                    likePost = likePostService.matchCommentary(commentaryId, user.getId());
                }
                  
                if(likePost == null){

                    likePost = createAndSave( postId,commentaryId, user);
                }

                likePost.setLike(like);
                likePostService.update(likePost);

                Post p = null;

                if(likePost != null){
                    p = likePost.getPost();
                }
                if(p == null && likePost != null && likePost.getCommentary() != null){
                    p = likePost.getCommentary().getPost();
                }


                return pb.createPostTO(p,id);

            }
        }
    }
}
