package com.tfg.api.projecte_tfg.rest;

import com.tfg.api.projecte_tfg.builder.NotificationsBuilder;
import com.tfg.api.projecte_tfg.enums.Status;
import com.tfg.api.projecte_tfg.model.Notifications;
import com.tfg.api.projecte_tfg.rest.object.ListNotificationsTO;
import com.tfg.api.projecte_tfg.rest.object.ListTO;
import com.tfg.api.projecte_tfg.rest.object.ModelJSON;
import com.tfg.api.projecte_tfg.rest.object.NotificationsTO;
import com.tfg.api.projecte_tfg.service.NotificationsService;

import javax.ejb.EJB;
import javax.enterprise.context.RequestScoped;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.ws.rs.*;
import javax.ws.rs.core.Context;
import java.util.List;

/**
 * Created by marc on 28/04/14.
 */

@Path("/notifications")
@RequestScoped
public class NotificationsRESTService {

    private static Integer MAX_PAGE = 8;

    @EJB
    private NotificationsService notificationsService;

    @EJB
    private NotificationsBuilder notificationsBuilder;


    @GET
    @Path("/my/{page}")
    @Produces("application/json")
    public Object getNotifications(
            @Context HttpServletRequest req,
            @PathParam("page") Long page) throws Exception {

        HttpSession session = req.getSession();

        if (session == null) {
            return new NotificationsTO(ModelJSON.ERROR,"Error: Error de session",null);
        }

        if (session.getAttribute("web_auth_id") == null) {
            return new NotificationsTO(ModelJSON.ERROR,"Error: Sessio no iniciada",null);
        }

        Long id = (Long) session.getAttribute("web_auth_id");

        try{

            page = page == null? 0 : page;

            List<Notifications> notificationsList = notificationsService.getNotificationsList(id,MAX_PAGE,page);

            Long n = notificationsService.getCount(id);
            Long nPage = 0l;
            if(n > 0){
                nPage = (n / MAX_PAGE) + 1;
            }


            ListNotificationsTO<NotificationsTO> notificationsListTO = notificationsBuilder.makeListTO(notificationsList,id,page,nPage);

            return notificationsListTO;
        }catch (Exception e){

            return new NotificationsTO(ModelJSON.ERROR,"Error: error inesperat.",null);
        }
    }

    @GET
    @Produces("application/json")
    public Object get(
            @Context HttpServletRequest req,
            @PathParam("id") Long idNotification) throws Exception {

        HttpSession session = req.getSession();

        if (session == null) {
            return new NotificationsTO(ModelJSON.ERROR,"Error: Error de session",null);
        }

        if (session.getAttribute("web_auth_id") == null) {
            return new NotificationsTO(ModelJSON.ERROR,"Error: Sessio no iniciada",null);
        }

        Long id = (Long) session.getAttribute("web_auth_id");

        try{

            Notifications notifications = notificationsService.get(idNotification);


            return notificationsBuilder.getNotificationsTO(id,notifications);

        }catch (Exception e){

            return new NotificationsTO(ModelJSON.ERROR,"Error: error inesperat.",null);
        }
    }

    @PUT
    @Path("{id}")
    @Produces("application/json")
    public Object modifyNotification(@Context HttpServletRequest req,
                              @PathParam("id") Long idNotification,
                              @FormParam("option") Boolean option,
                              @FormParam("isnew") Boolean isNew) throws Exception {

        // Access to the HTTP session
        HttpSession session = req.getSession();

        if (session == null) {
            return new NotificationsTO(ModelJSON.ERROR,"Error: Error de session",null);
        }

        if (session.getAttribute("web_auth_id") == null) {
            return new NotificationsTO(ModelJSON.ERROR,"Error: Sessio no iniciada",null);
        }

        Long id = (Long) session.getAttribute("web_auth_id");

        try{

            Notifications notifications = notificationsService.getNotifications(idNotification);

            if(isNew != null && !isNew){
                notifications.setIsNew(false);
            }

            if(Status.PENDING.equals(notifications.getStatus())){
                if(option != null){
                    if(option){
                        notifications.setStatus(Status.OK);
                        notifications.setIsNew(false);
                        notifications.setDescription("T'has unit al grup ");
                        notificationsService.update(notifications);

                    }else{
                        notifications.setStatus(Status.NO);
                        notifications.setIsNew(false);
                        ///notificationsService.delete(notifications);
                        notificationsService.update(notifications);

                    }

                    notificationsBuilder.notificationResponse(notifications,option,id);
                }
            }else{
                notificationsService.update(notifications);
            }


            return new ModelJSON(ModelJSON.OK,ModelJSON.MSG_EMPTY,id);

        }catch (Exception e){

            return new NotificationsTO(ModelJSON.ERROR,"Error: error inesperat.",null);

        }
    }
}
