package com.tfg.api.projecte_tfg.rest;

import com.tfg.api.projecte_tfg.builder.FriendBuilder;
import com.tfg.api.projecte_tfg.builder.PostBuilder;
import com.tfg.api.projecte_tfg.builder.UserBuilder;
import com.tfg.api.projecte_tfg.enums.Relation;
import com.tfg.api.projecte_tfg.enums.Status;
import com.tfg.api.projecte_tfg.enums.TypeNotifications;
import com.tfg.api.projecte_tfg.enums.TypePost;
import com.tfg.api.projecte_tfg.model.*;
import com.tfg.api.projecte_tfg.rest.object.ListTO;
import com.tfg.api.projecte_tfg.rest.object.ModelJSON;
import com.tfg.api.projecte_tfg.rest.object.PostTO;
import com.tfg.api.projecte_tfg.rest.object.UserTO;
import com.tfg.api.projecte_tfg.service.*;
import com.tfg.api.projecte_tfg.util.StringUtils;

import javax.ejb.EJB;
import javax.enterprise.context.RequestScoped;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.ws.rs.*;
import javax.ws.rs.core.Context;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

/**
 * Created by marc on 26/03/14.
 */

@Path("/post")
@RequestScoped
public class PostRESTService {

    private static Integer MAX_POST = 15;

    @EJB
    private PostService ps;

    @EJB
    private PostBuilder pb;

    @EJB
    private UserService us;

    @EJB
    private UserBuilder userBuilder;

    @EJB
    private FriendBuilder friendBuilder;

    @EJB
    private GrupService grupService;

    @EJB
    private NotificationsService notificationsService;
    @EJB
    private HashtagService hashtagService;
    @EJB
    private PostHashTagService postHashTagService;


    @GET
    @Path("/my")
    @Produces("application/json")
    public ListTO<PostTO> getPost(@Context HttpServletRequest req) throws Exception {

        // Access to the HTTP session
        HttpSession session = req.getSession();

        if (isSessionError(session)) return new ListTO<PostTO>(ModelJSON.ERROR, "Error: Error de session", null, null);

        if (session.getAttribute("web_auth_id") == null) {
            return new ListTO<PostTO>(ModelJSON.ERROR, ModelJSON.MSG_EMPTY,null,null);
        }

            Long id = (Long) session.getAttribute("web_auth_id");
            List<Post> lp = ps.getByIdUser(id);

            if (lp == null || lp.isEmpty()){
                return new ListTO<PostTO>(ModelJSON.ERROR, ModelJSON.MSG_EMPTY,null,null);
            }else{

                return getPostTOListTO(id, lp);
            }
    }

    @GET
    @Path("/user/{id}/{lastId}")
    @Produces("application/json")
    public ListTO<PostTO> getListUserPostPagination(@Context HttpServletRequest req,
                                          @PathParam("id") Long idUser,
                                          @PathParam("lastId") Long lastId) throws Exception {

        // Access to the HTTP session
        HttpSession session = req.getSession();

        if (isSessionError(session)) return new ListTO<PostTO>(ModelJSON.ERROR, "Error: Error de session", null, null);

        if (session.getAttribute("web_auth_id") == null) {
            return new ListTO<PostTO>(ModelJSON.ERROR, ModelJSON.MSG_EMPTY,null,null);
        } else {

            Long id = (Long) session.getAttribute("web_auth_id");
            if(idUser == null){
                return new ListTO<PostTO>(ModelJSON.ERROR, "User id Null",null,null);
            }

            Relation r = friendBuilder.friend(id,idUser);

            List<Post> lp = new ArrayList<Post>();

            if(Relation.OWN.equals(r) || Relation.FRIEND.equals(r)){
                Integer MAX_POST = 20;

                if(lastId == null || lastId == -1){
                    lp = ps.getByIdUserToandFrom(idUser,MAX_POST);

                }else{
                    lp = ps.getByIdUserToandFrom(idUser,MAX_POST,lastId);

                }

            }

            if (lp == null || lp.isEmpty()){
                return new ListTO<PostTO>(ModelJSON.ERROR,"Not friend",null,null);
            }else{
                return getPostTOListTO(id, lp);
            }
        }

    }

    @GET
    @Path("/{id}")
    @Produces("application/json")
    public ListTO<PostTO> getListUserPost(@Context HttpServletRequest req,
                                     @PathParam("id") Long userId) throws Exception {

        // Access to the HTTP session
        HttpSession session = req.getSession();

        if (isSessionError(session)) return new ListTO<PostTO>(ModelJSON.ERROR, "Error: Error de session", null, null);

        if (session.getAttribute("web_auth_id") == null) {
            return new ListTO<PostTO>(ModelJSON.ERROR, ModelJSON.MSG_EMPTY,null,null);
        } else {

            Long id = (Long) session.getAttribute("web_auth_id");
            if(userId == null){
                return new ListTO<PostTO>(ModelJSON.ERROR, "User id Null",null,null);
            }

            Relation r = friendBuilder.friend(id,userId);

            List<Post> lp = new ArrayList<Post>();

            if(Relation.OWN.equals(r) || Relation.FRIEND.equals(r)){
                Integer MAX_POST = 20;

                lp = ps.getPostByUserId(userId);

            }

            if (lp == null || lp.isEmpty()){
                return new ListTO<PostTO>(ModelJSON.ERROR,"Not friend",null,null);
            }else{
                return getPostTOListTO(id, lp);
            }
        }

    }

    @GET
    @Path("/hashtag/{hashtag}")
    @Produces("application/json")
    public ListTO<PostTO> getAllPostByHashTag(@Context HttpServletRequest req,
                                     @PathParam("hashtag") String hashtag) throws Exception {

        // Access to the HTTP session
        HttpSession session = req.getSession();

        if (isSessionError(session)) return new ListTO<PostTO>(ModelJSON.ERROR, "Error: Error de session", null, null);

        if (session.getAttribute("web_auth_id") == null) {
            return new ListTO<PostTO>(ModelJSON.ERROR, ModelJSON.MSG_EMPTY,null,null);
        } else {

            Long id = (Long) session.getAttribute("web_auth_id");
            if(hashtag == null){
                return new ListTO<PostTO>(ModelJSON.ERROR, "User id Null",null,null);
            }

            hashtag = hashtag.trim();

            List<Post> lp = postHashTagService.getTop50ByHashTag(hashtag);


            if (lp == null || lp.isEmpty()){
                return new ListTO<PostTO>(ModelJSON.ERROR,"Not friend",null,null);
            }else{
                return getPostTOListTO(id, lp);
            }
        }

    }

    @GET
    @Path("/group/{id}/{lastId}")
    @Produces("application/json")
    public ListTO<PostTO> getListGrupPostPagination(@Context HttpServletRequest req,
                                           @PathParam("id") Long groupId,
                                           @PathParam("lastId") Long lastId) throws Exception {

        // Access to the HTTP session
        HttpSession session = req.getSession();

        if (isSessionError(session)) return new ListTO<PostTO>(ModelJSON.ERROR, "Error: Error de session", null, null);

        if (session.getAttribute("web_auth_id") == null) {
            return new ListTO<PostTO>(ModelJSON.ERROR, ModelJSON.MSG_EMPTY,null,null);
        } else {

            Long id = (Long) session.getAttribute("web_auth_id");
            if(groupId == null){
                return new ListTO<PostTO>(ModelJSON.ERROR, "User id Null",null,null);
            }

            List<Post> lp = new ArrayList<Post>();

            Integer MAX_POST = 20;

            if(lastId == null || lastId == -1){
                lp = ps.getByIdGroupToandFrom(groupId, MAX_POST);

            }else{
                lp = ps.getByIdGroupToandFrom(groupId, MAX_POST, lastId);
            }

            if (lp == null || lp.isEmpty()){
                return new ListTO<PostTO>(ModelJSON.ERROR, ModelJSON.MSG_EMPTY,null,null);
            }else{
                return getPostTOListTO(id, lp);
            }
        }

    }

    private ListTO<PostTO> getPostTOListTO(Long id, List<Post> lp) {
        ListTO<PostTO> postListTO = new ListTO<PostTO>(id);

        List<PostTO> postTOList = new ArrayList<PostTO>(lp.size());

        for(Post p: lp){
            postTOList.add(pb.createPostTO(p,id));

        }
        postListTO.setList(postTOList);

        return postListTO;
    }

    @GET
    @Path("/friends/{id}/{lastId}")
    @Produces("application/json")
    public ListTO<PostTO> getListFriendsPostPagination(@Context HttpServletRequest req,
                                         @PathParam("id") Long userId,
                                         @PathParam("lastId") Long lastId) throws Exception {

        // Access to the HTTP session
        HttpSession session = req.getSession();

        if (isSessionError(session)) return new ListTO<PostTO>(ModelJSON.ERROR, "Error: Error de session", null, null);

        if (session.getAttribute("web_auth_id") == null) {
            return new ListTO<PostTO>(ModelJSON.ERROR, ModelJSON.MSG_EMPTY,null,null);
        } else {

            Long id = (Long) session.getAttribute("web_auth_id");
            List<Post> lp = null;

            Relation r = friendBuilder.friend(id,userId);

                if(Relation.OWN.equals(r) || Relation.FRIEND.equals(r)){
                    Integer MAX_POST = 20;

                if(lastId == null || lastId == -1){
                    lp = ps.getPostFriendsByIdUser(userId, MAX_POST);

                }else{
                    lp = ps.getPostFriendsByIdUser(userId, MAX_POST, lastId);
                }

                if (lp == null || lp.isEmpty()){
                    return new ListTO<PostTO>(ModelJSON.ERROR, ModelJSON.MSG_EMPTY,null,null);
                }else{
                    return getPostTOListTO(id, lp);
                }
            }{

                return new ListTO<PostTO>(ModelJSON.ERROR, "Not friend",null,null);

            }
        }
    }

    @POST
    @Produces("application/json")
    public PostTO createPost(@Context HttpServletRequest req,
                             @FormParam("text") String text,
                             @FormParam("userToId") Long userToId,
                             @FormParam("grupToId") Long grupToId,
                             @FormParam("userFromId") Long userFromId,
                             @FormParam("groupFromId") Long groupFromId,
                             @FormParam("postShareId") Long postShareId,
                             @FormParam("scTrackId") Long scTrackId,
                             @FormParam("url") String url) throws Exception {

        // Access to the HTTP session
        HttpSession session = req.getSession();

        if (isSessionError(session)) return pb.createError("Error: Error de session");

        if (session.getAttribute("web_auth_id") == null) {
            return pb.createError(ModelJSON.MSG_EMPTY);
        } else {

            Long id = (Long) session.getAttribute("web_auth_id");

            User userFrom = us.getUser(userFromId);
            User userto = us.getUser(userToId);
            Grup grup_from = grupService.getGrup(groupFromId);
            Grup grup_to = grupService.getGrup(grupToId);
            Post postShare = null;
            SCTrack scTrack = null;


            Post p = new Post(text, false, userto,
                    grup_to, userFrom, grup_from, postShare,scTrack);
            p = ps.save(p);

            //Check hashTags
            checkHashTag(p);

            if (p == null){
                return pb.createError(ModelJSON.MSG_EMPTY);

            }else{

                if(p.getUser_to() != null && !p.getUser_to().getId().equals(id)){
                    Notifications notifications = notificationsService.save(new Notifications(
                            TypeNotifications.FRIEND_POST, true, Status.OK,
                            "a fet una publicació en el teu perfil.", p.getUser_to(),
                            p.getGrup_to(), p.getUser_from(), p.getGrup_from()));
                }

                return pb.createPostTO(p,id);
            }
        }

    }


    @POST
    @Path("/share")
    @Produces("application/json")
    public PostTO sharePost(@Context HttpServletRequest req,
                             @FormParam("id") Long idPost) throws Exception {

        // Access to the HTTP session
        HttpSession session = req.getSession();

        if (isSessionError(session)) return pb.createError("Error: Error de session");

        if (session.getAttribute("web_auth_id") == null) {
            return pb.createError(ModelJSON.MSG_EMPTY);
        } else {

            Long id = (Long) session.getAttribute("web_auth_id");
            User ucurrent = us.getUser(id);

            Post pshared = ps.find(idPost);

            User userFrom = ucurrent;
            User userto = null;
            Grup grup_from = null;
            Grup grup_to = null;
            Post postShare = pshared;
            SCTrack scTrack = null;


            Post p = new Post(pshared.getText(), false, userto,
                    grup_to, userFrom, grup_from, postShare,scTrack);
            p = ps.save(p);

            if (p == null){
                return pb.createError(ModelJSON.MSG_EMPTY);

            }else{

                return pb.createPostTO(p,id);
            }
        }

    }

    private boolean isSessionError(HttpSession session) {
        if (session == null) {
            return true;
        }
        return false;
    }

    @DELETE
    @Path("/my/{id}")
    @Produces("application/json")
    public PostTO deletePost(@Context HttpServletRequest req,
                          @PathParam("id") Long idPost) throws Exception {

        // Access to the HTTP session
        HttpSession session = req.getSession();

        if (isSessionError(session)) return pb.createError("Error: Error de session");

        if (session.getAttribute("web_auth_id") == null) {
            return pb.createError(ModelJSON.MSG_EMPTY);
        } else {

            Long id = (Long) session.getAttribute("web_auth_id");
            User u = us.getUser(id);
            Post p = ps.find(idPost);

            if (p == null){
                return pb.createError(ModelJSON.MSG_EMPTY);

            }else{
                try {
                    p.setDelete(true);
                    ps.update(p);
                    return pb.createOKNull(id);

                }   catch (Exception e){
                    return pb.createError(ModelJSON.MSG_EMPTY);
                }

            }
        }

    }


    private void checkHashTag(Post post){

        if(post == null){return;}

        String text = post.getText();
        if(StringUtils.notEmptyChar(text)){

            String[] wordsList = text.split(" ");

            for(String s : wordsList){

                if(StringUtils.notEmptyChar(s) &&
                        s.subSequence(0,1).equals("#") &&
                        s.length() > 1){

                    s = s.trim();
                    Hashtag hashtag = hashtagService.getByHashTag(s);

                    if(hashtag == null){
                        hashtag = new Hashtag(s);
                        hashtagService.save(hashtag);
                    }

                    PostHashTag postHashTag = postHashTagService.match(hashtag,post);

                    if(postHashTag == null){
                        postHashTag = postHashTagService.save(new PostHashTag(post,hashtag));
                    }
                }
            }
        }
    }
}
