package com.tfg.api.projecte_tfg.rest;

import com.tfg.api.projecte_tfg.builder.SouncloudBuilder;
import com.tfg.api.projecte_tfg.enums.Member;
import com.tfg.api.projecte_tfg.enums.Status;
import com.tfg.api.projecte_tfg.enums.TypeNotifications;
import com.tfg.api.projecte_tfg.model.*;
import com.tfg.api.projecte_tfg.rest.object.ListTO;
import com.tfg.api.projecte_tfg.rest.object.ModelJSON;
import com.tfg.api.projecte_tfg.rest.object.PostTO;
import com.tfg.api.projecte_tfg.rest.object.SCTrackTO;
import com.tfg.api.projecte_tfg.service.*;

import javax.ejb.EJB;
import javax.enterprise.context.RequestScoped;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.ws.rs.*;
import javax.ws.rs.core.Context;
import java.util.List;

/**
 * Created by marc on 12/05/14.
 */
@Path("/soundcloud")
@RequestScoped
public class SCTrackRESTService {

    @EJB
    private SouncloudBuilder souncloudBuilder;
    @EJB
    private GrupService grupService;
    @EJB
    private SCTrackService scTrackService;
    @EJB
    private PostService postService;
    @EJB
    private UserService userService;
    @EJB
    private Grup_UserService grup_userService;


    private boolean isSessionError(HttpSession session) {
        if (session == null) {
            return true;
        }
        return false;
    }

    @POST
    @Produces("application/json")
    public ListTO<SCTrackTO> createSong(@Context HttpServletRequest req,
                                        @FormParam("grupId") Long grupId,
                                        @FormParam("name") String name,
                                        @FormParam("iframe") String iframe,
                                        @FormParam("url") String url) throws Exception {

        // Access to the HTTP session
        HttpSession session = req.getSession();

        if (isSessionError(session)) return souncloudBuilder.createError("Error: Error de session");

        if (session.getAttribute("web_auth_id") == null) {
            return souncloudBuilder.createError(ModelJSON.MSG_EMPTY);
        } else {

            Long id = (Long) session.getAttribute("web_auth_id");

            Grup grup = grupService.getGrup(grupId);
            User user = userService.getUser(id);


            if(grup != null && user != null){

                if(isMemberAdminOrFounder(grup,user)){

                    SCTrack song = createTrack(name, iframe, url, grup);

                    createPost(grup, song);

                    return createListTracksTO(id, grup);
                }
            }
        }

        return souncloudBuilder.createError("Error: al crear la cançó");

    }

    private SCTrack createTrack(String name, String iframe, String url, Grup grup) {
        SCTrack song = new SCTrack(grup,name,iframe,url);
        scTrackService.save(song);
        return song;
    }

    private ListTO<SCTrackTO> createListTracksTO(Long id, Grup grup) {
        Integer MAX = 5;
        Long page = 0l;
        List<SCTrack> scTrackList = scTrackService.getListByGrup(grup.getId(),MAX,page);
        List<SCTrackTO> scTrackTOList = souncloudBuilder.createListTO(scTrackList, id);
        return new ListTO<SCTrackTO>(ModelJSON.OK,ModelJSON.MSG_EMPTY,id,scTrackTOList);
    }

    private boolean isMemberAdminOrFounder(Grup grup, User user) {
        GrupMember grupMember = grup_userService.match(grup.getId(), user.getId());

        return Status.OK.equals(grupMember.getStatus())
                && (Member.FOUNDER.equals(grupMember.getTypemember())
                || Member.ADMIN.equals(grupMember.getTypemember()));
    }

    @GET
    @Path("/top4")
    @Produces("application/json")
    public ListTO<SCTrackTO> topFour(@Context HttpServletRequest req) throws Exception {

        // Access to the HTTP session
        HttpSession session = req.getSession();

        if (isSessionError(session)) return souncloudBuilder.createError("Error: Error de session");

            Long id = session.getAttribute("web_auth_id") == null ? null :
                    (Long) session.getAttribute("web_auth_id");

            List<SCTrack> scTrackList = scTrackService.top4();

            if(scTrackList == null || scTrackList.size() < 4){
                List<SCTrack> scTrackList2 = scTrackService.top4Alltime();

                int i = 0;
                while (scTrackList.size() < 4 &&
                        scTrackList2 != null &&
                        i < scTrackList2.size()){

                    if(notHave(scTrackList,scTrackList2.get(i))){
                        scTrackList.add(scTrackList2.get(i));
                    }
                    i++;
                }


            }

            List<SCTrackTO> scTrackTOList = souncloudBuilder.createListTO(scTrackList, id);



            return new ListTO<SCTrackTO>(ModelJSON.OK,ModelJSON.MSG_EMPTY,id,scTrackTOList);

    }

    private boolean notHave(List<SCTrack> scTrackList, SCTrack scTrack) {

        for(SCTrack scTrack1 : scTrackList){

            if(scTrack1.getId().equals(scTrack.getId())){
                return false;
            }

        }
        return true;
    }

    private void createPost(Grup grup, SCTrack song) {

        Post post = new Post(song.getName(),false,null,null,null,grup,null,song);
        postService.save(post);

    }


}
