package com.tfg.api.projecte_tfg.rest;

import com.tfg.api.projecte_tfg.builder.SearchBuilder;
import com.tfg.api.projecte_tfg.builder.UserBuilder;
import com.tfg.api.projecte_tfg.model.*;
import com.tfg.api.projecte_tfg.rest.object.ListTO;
import com.tfg.api.projecte_tfg.rest.object.ModelJSON;
import com.tfg.api.projecte_tfg.rest.object.SearchItemTO;
import com.tfg.api.projecte_tfg.rest.object.UserTO;
import com.tfg.api.projecte_tfg.service.*;
import com.tfg.api.projecte_tfg.util.StringUtils;

import javax.ejb.EJB;
import javax.enterprise.context.RequestScoped;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

/**
 * Created by marc on 01/04/14.
 */

@Path("/search")
@RequestScoped
public class SearchRESTService {

    @EJB
    UserService us;
    @EJB
    PostService ps;
    @EJB
    UserBuilder ub;
    @EJB
    GrupService grupService;
    @EJB
    SearchBuilder searchBuilder;
    @EJB
    private HashtagService hashtagService;
    @EJB
    private PostHashTagService postHashTagService;


    @GET
    @Path("/{text}")
    @Produces("application/json")
    public SearchItemTO getSearch(@Context HttpServletRequest req,
                                  @PathParam("text") String textSearch) throws Exception {

        // Access to the HTTP session
        HttpSession session = req.getSession();

        if (session == null) {
            return searchBuilder.error("Error: Error de session");
        }

        if (session.getAttribute("web_auth_id") == null) {
            return searchBuilder.error(ModelJSON.MSG_EMPTY);
        }else if (StringUtils.isEmptyChar(textSearch)){
            return searchBuilder.error("Empty Text");
        }else{

            List<Grup> grupList = grupService.getGrupByTextLike(textSearch);
            List<User> list = us.getUserByTextLike(textSearch);
            List<Post> lp = new LinkedList<Post>();

            List<String> hashtags = new LinkedList<String>();

            for(String hashtag : textSearch.split(" ")){
                if(StringUtils.notEmptyChar(hashtag)){
                    hashtags.addAll(hashtagService.getListHashtagsLike(hashtag));
                    hashtag = hashtag.trim();

                    lp.addAll(postHashTagService.getTop50ByHashTag(hashtag));
                }
            }

            return searchBuilder.createObjectListSearch(grupList, list,lp,hashtags, (Long) session.getAttribute("web_auth_id"));

        }

    }

    @GET
    @Path("/friends/{text}")
    @Produces("application/json")
    public ListTO<UserTO> getUsersFreinds(@Context HttpServletRequest req,
                                  @PathParam("text") String textSearch) throws Exception {

        // Access to the HTTP session
        HttpSession session = req.getSession();

        if (session == null) {
            return new ListTO<UserTO>(ModelJSON.ERROR, "Error: Error de session",null,null);
        }

        if (session.getAttribute("web_auth_id") == null) {
            return new ListTO<UserTO>(ModelJSON.ERROR, ModelJSON.MSG_EMPTY,null,null);
        }else if (StringUtils.isEmptyChar(textSearch)){
            return new ListTO<UserTO>(ModelJSON.ERROR, "Empty Text",null,null);
        }else{

            Long id = (Long) session.getAttribute("web_auth_id");
            List<User> friendList = us.getFriendsrByTextLikeTop5(id,textSearch);
            //List<User> userList = new ArrayList<User>(friendList.size());

            //for(Friend f: friendList)
            //    userList.add(f.getUser2());

            return ub.ListSearch(friendList, (Long) session.getAttribute("web_auth_id"));

        }

    }
}
