package com.tfg.api.projecte_tfg.service;

import com.tfg.api.projecte_tfg.enums.Status;
import com.tfg.api.projecte_tfg.model.Friend;
import com.tfg.api.projecte_tfg.model.InfoUser;
import com.tfg.api.projecte_tfg.model.User;

import javax.ejb.LocalBean;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import java.util.List;

/**
 * Created by marc on 01/04/14.
 */

@Stateless
@LocalBean
public class FriendService {

    @PersistenceContext
    private EntityManager em;

    public Friend match(Long id1, Long id2) {

        try {

            Query q = em.createQuery("select f from Friend f where (f.user1.id=:id1 and f.user2.id=:id2)");

            q.setParameter("id1", id1);
            q.setParameter("id2", id2);

            return singleResult(q);
        }
        catch (Exception e) {
            return null;
        }

    }


    private Friend singleResult(Query q){

        List<Friend> l = q.getResultList();

        if(l != null && l.size() == 1){
            return l.get(0);
        }else if(l != null && l.size() > 1){
            return l.get(0);
        }else return null;
    }


    //Script:
    //select count(*) from Friend f where f.IdUSER1 = '5' AND f.IdUSER2 IN
    // (select f2.IdUSER2 from Friend f2 where f2.IdUSER1='1');
    public Long getCommonFriends(Long id1, Long id2) {

        try {
            Query q = em.createQuery("select count(f) from Friend f where f.user1.id = :id2  and f.status =:status " +
                    " AND f.user2.id IN (select f2.user2.id from Friend f2 where f2.user1.id=:id1 and f2.status =:status)");

            q.setParameter("id1", id1);
            q.setParameter("id2", id2);
            q.setParameter("status", Status.OK);

            Long i = (Long) q.getSingleResult();
            return i;
        }
        catch (Exception e) {
            return null;
        }


    }

    /*
    retorna el numero de Usuaris que segueix l'usuari amb id :id
     */
    public Long getFollows(Long id){

        try{
            Query q = em.createQuery("select count(f) from Friend f where f.user1.id = :id and f.status =:status");

            q.setParameter("id", id);
            q.setParameter("status", Status.OK);

            Long i = (Long) q.getSingleResult();
            return i;

        }catch (Exception e ){

        }
        return null;
    }

    /*
    retorna el numero de Usuaris que segueix l'usuaria amb id :id
     */
    public Long getFollowers(Long id){

        try{
            Query q = em.createQuery("select count(f) from Friend f where f.user2.id = :id and f.status =:status");

            q.setParameter("id", id);
            q.setParameter("status", Status.OK);

            Long i = (Long) q.getSingleResult();
            return i;

        }catch (Exception e ){

        }
        return null;
    }

    public Friend save(Friend friend) {

        try {
            em.persist(friend);
            em.flush();
            return friend;

        }   catch (Exception e){
            return null;
        }

    }

    public Friend update(Friend f) {

        try {
            em.merge(f);
            em.flush();
            return f;

        }   catch (Exception e){
            return null;
        }
    }

    public List<Friend> getAllFollowers(Long userId) {
        try{
            Query q = em.createQuery("select f from Friend f where f.user2.id = :id and f.status =:status");

            q.setParameter("id", userId);
            q.setParameter("status", Status.OK);

            return (List<Friend>) q.getResultList();

        }catch (Exception e ){

        }
        return null;
    }

    public List<Friend> getAllFollows(Long userId) {
        try{
            Query q = em.createQuery("select f from Friend f where f.user1.id = :id and f.status =:status");

            q.setParameter("id", userId);
            q.setParameter("status", Status.OK);

            return (List<Friend>) q.getResultList();

        }catch (Exception e ){

        }
        return null;
    }
}
