package com.tfg.api.projecte_tfg.service;

import com.tfg.api.projecte_tfg.model.Hashtag;
import com.tfg.api.projecte_tfg.model.Post;
import com.tfg.api.projecte_tfg.model.PostHashTag;
import com.tfg.api.projecte_tfg.util.DateUtils;
import com.tfg.api.projecte_tfg.util.MapUtils;

import javax.ejb.LocalBean;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import java.util.*;

/**
 * Created by marc on 09/05/14.
 */
@Stateless
@LocalBean
public class PostHashTagService {

    @PersistenceContext
    private EntityManager em;

    public PostHashTag match(Hashtag hashtag, Post post) {
        try{
            Query q = em.createQuery("select ph from PostHashTag ph " +
                    "where ph.post =:postId and ph.hashtag.id =:hashtagId");

            q.setParameter("postId", post.getId());
            q.setParameter("hashtagId", hashtag.getId());

            return (PostHashTag) q.getSingleResult();

        }catch (Exception e){
            return null;
        }
    }

    public PostHashTag save(PostHashTag postHashTag) {

        em.persist(postHashTag);
        return postHashTag;
    }

    public List<Post> getTop50ByHashTag(String hashtag) {

        try{
            hashtag = hashtag.trim();

            if(!hashtag.substring(0,1).equals("#")){
                hashtag = "#" + hashtag;
            }

            Query q = em.createQuery("select ph.post from PostHashTag ph " +
                    "where ph.hashtag.hashtag =:hashtag order by ph.createdAt desc");

            q.setParameter("hashtag", hashtag);
            q.setMaxResults(50);

            return (List<Post>) q.getResultList();

        }catch (Exception e){
            return null;
        }
    }

    public List<String> getTodayTendency() {

        try{

            Query q = em.createQuery("select ph.hashtag.hashtag from PostHashTag ph " +
                    "where ph.createdAt > :today group by ph.hashtag.id order by count(ph.hashtag.id) desc ");

            q.setParameter("today", DateUtils.get24hAgo());

            q.setMaxResults(10);

            return (List<String>) q.getResultList();

        }catch (Exception e){
            return null;
        }
    }
}
