package com.tfg.api.projecte_tfg.service;

import com.tfg.api.projecte_tfg.enums.Status;
import com.tfg.api.projecte_tfg.model.InfoUser;
import com.tfg.api.projecte_tfg.model.Post;

import javax.ejb.LocalBean;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import java.util.List;

/**
 * Created by marc on 26/03/14.
 */

@Stateless
@LocalBean
public class PostService {

    @PersistenceContext
    private EntityManager em;

    public List<Post> getByIdUser(Long id) {

        try {
            Query q = em.createQuery("select p from Post p where (p.user_from.id=:id OR p.user_to=:id)" +
                    " and (p.delete = false) order by p.createdAt desc ");
            q.setParameter("id", id);
            return (List<Post>) q.getResultList();
        }
        catch (Exception e) {
            return null;
        }
    }

    public Post save(Post p) {

        try{
            em.persist(p);
            return p;

        }   catch (Exception e){
            return null;
        }
    }

    public Post find(Long idPost) {
        return em.find(Post.class,idPost);
    }

    public void update(Post p) {
        em.merge(p);
    }

    public List<Post> getByIdUserToandFrom(Long id,Integer maxResult) {

        try {
            Query q = em.createQuery("select p from Post p where (p.user_from.id=:id OR p.user_to=:id)" +
                    " and (p.delete = false) order by p.createdAt desc ");

            if(maxResult != null){
                q.setMaxResults(maxResult);
            }

            q.setParameter("id", id);
            return (List<Post>) q.getResultList();
        }
        catch (Exception e) {
            return null;
        }

    }

    public List<Post> getByIdUserToandFrom(Long id,Integer maxResult,Long lastPostId) {

        try {
            Query q = em.createQuery("select p from Post p where (p.user_from.id=:id OR p.user_to=:id)" +
                    " and (p.delete = false) and p.id < :lastPostId  order by p.createdAt desc ");

            if(maxResult != null){
                q.setMaxResults(maxResult);
            }

            q.setParameter("id", id);
            q.setParameter("lastPostId", lastPostId);
            return (List<Post>) q.getResultList();
        }
        catch (Exception e) {
            return null;
        }

    }

    public List<Post> getPostFriendsByIdUser(Long id) {

        try {
            Query q = em.createQuery("select p from Post p where (p.user_from.id=:id " +
                    "OR p.user_from.id in (select f.user2.id from Friend f where f.user1.id=:id)" +
                    "OR p.user_to.id=:id) and (p.delete = false) order by p.createdAt desc ");

            q.setParameter("id", id);
            return (List<Post>) q.getResultList();
        }
        catch (Exception e) {
            return null;
        }


    }

    public Long getTotalPostFromUser(Long id) {
        try {
            Query q = em.createQuery("select count(p) from Post p where p.user_from.id=:id and (p.delete = false)");
            q.setParameter("id", id);
            return (Long) q.getSingleResult();
        }
        catch (Exception e) {
            return null;
        }
    }

    public List<Post> getByIdGroupToandFrom(Long groupId, Integer MAX_POST) {

        try {
            Query q = em.createQuery("select p from Post p where (p.grup_from.id=:id OR p.grup_to=:id)" +
                    " and (p.delete = false) order by p.createdAt desc ");


            if(MAX_POST != null){
                q.setMaxResults(MAX_POST);
            }

            q.setParameter("id", groupId);
            return (List<Post>) q.getResultList();
        }
        catch (Exception e) {
            return null;
        }
    }

    public List<Post> getByIdGroupToandFrom(Long groupId, Integer MAX_POST,Long lastPostId) {

        try {
            Query q = em.createQuery("select p from Post p where (p.grup_from.id=:id OR p.grup_to=:id)" +
                    " and (p.delete = false) and p.id < :lastPostId order by p.createdAt desc ");


            if(MAX_POST != null){
                q.setMaxResults(MAX_POST);
            }

            q.setParameter("lastPostId", lastPostId);
            q.setParameter("id", groupId);
            return (List<Post>) q.getResultList();
        }
        catch (Exception e) {
            return null;
        }
    }

    public Post get(Long postId) {
        return em.find(Post.class,postId);
    }

    public List<Post> getPostFriendsByIdUser(Long userId, Integer MAX_POST, Long lastPostId) {
        try {
            Query q = em.createQuery("select p from Post p where (p.user_from.id=:id " +
                    "OR p.user_from.id in (select f.user2.id from Friend f where f.user1.id=:id and f.status=:status)" +
                    "OR p.user_to.id=:id " +
                    "OR p.grup_from.id in (select g.grup.id from GrupMember g where g.user.id=:id and g.status=:status)" +
                    "OR p.grup_from.id in (select f.grup.id from FollowGrup f where f.user.id=:id and f.follow=true)) " +
                    "AND (p.delete = false) and p.id < :lastPostId order by p.createdAt desc ");

            q.setParameter("id", userId);
            q.setParameter("status", Status.OK);

            if(MAX_POST != null){
                q.setMaxResults(MAX_POST);
            }

            q.setParameter("lastPostId", lastPostId);

            return (List<Post>) q.getResultList();
        }
        catch (Exception e) {
            return null;
        }
    }

    public List<Post> getPostFriendsByIdUser(Long userId, Integer MAX_POST) {
        try {
            Query q = em.createQuery("select p from Post p where (p.user_from.id=:id " +
                    "OR p.user_from.id in (select f.user2.id from Friend f where f.user1.id=:id and f.status=:status)" +
                    "OR p.user_to.id=:id " +
                    "OR p.grup_from.id in (select g.grup.id from GrupMember g where g.user.id=:id and g.status=:status)" +
                    "OR p.grup_from.id in (select f.grup.id from FollowGrup f where f.user.id=:id and f.follow=true)) " +
                    "and (p.delete = false) order by p.createdAt desc ");

            q.setParameter("id", userId);
            q.setParameter("status", Status.OK);

            if(MAX_POST != null){
                q.setMaxResults(MAX_POST);
            }

            return (List<Post>) q.getResultList();
        }
        catch (Exception e) {
            return null;
        }


    }


    public List<Post> getPostByUserId(Long userId) {

        try {
            Query q = em.createQuery("select p from Post p where p.user_from.id=:id " +
                    " and (p.delete = false) order by p.createdAt desc ");

            q.setParameter("id", userId);

            return (List<Post>) q.getResultList();
        }
        catch (Exception e) {
            return null;
        }
    }

}
